#
# Copyright (c) 2000, 2023, trustbe and/or its affiliates. All rights reserved.
# TRUSTBE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
#
#

# https://github.com/pypa/pip/issues/3969
# or False if you want to disable it
manylinux1_compatible = True
